#
#    Copyright (c) 2018 - 2020 Analog Flavor EURL
#
#    Contact us for support:
#        http://www.analogflavor.com
#
# -----------------------------------------------------------------------------
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright notice,
#      this list of conditions and the following disclaimer.
#
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#
#    * Neither the name of Analog Flavor EURL nor 
#      the names of its contributors may be used to endorse or promote products derived from
#      this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# -----------------------------------------------------------------------------
#


## \file 0004-high-level-module-functions.py
#  \brief This file contains an example for the use of the WienerMpodLvHvSnmpPython module.
#
#  The used functions allow to control and monitor a single Wiener MPOD crate over SNMP.
#
#  The comments in the example code give more explanations on the performed API function calls.

import sys
import time
import math
import argparse
import import_mpod_snmp_module

def test_module_functions(mpod_module) :

    # command: getNumberOfChannels
    #     input: -
    #     output: integer value
    result = mpod_module.getNumberOfChannels()
    print("getNumberOfChannels() returns \"{0}\"".format(result))

    # command: getAuxiliaryMeasurementVoltage0
    #     input: -
    #     output: real
    result = mpod_module.getAuxiliaryMeasurementVoltage0()
    print("getAuxiliaryMeasurementVoltage0() returns \"{0}\"".format(result))

    # command: getAuxiliaryMeasurementVoltage1
    #     input: -
    #     output: real
    result = mpod_module.getAuxiliaryMeasurementVoltage1()
    print("getAuxiliaryMeasurementVoltage1() returns \"{0}\"".format(result))

    # command: getDescription
    #     input: -
    #     output: string
    result = mpod_module.getDescription()
    print("getDescription() returns \"{0}\"".format(result))

    # command: getEventChannelStatus
    #     input: -
    #     output: integer
    result = mpod_module.getEventChannelStatus()
    print("getEventChannelStatus() returns \"{0}\"".format(result))

    # command: getEventStatus
    #     input: -
    #     output: integer
    result = mpod_module.getEventStatus()
    print("getEventStatus() returns \"{0}\"".format(result))

    # command: getEventStatusAsString
    #     input: -
    #     output: string
    result = mpod_module.getEventStatusAsString()
    print("getEventStatusAsString() returns \"{0}\"".format(result))

    # command: getHardwareLimitCurrent
    #     input: -
    #     output: real value
    result = mpod_module.getHardwareLimitCurrent()
    print("getHardwareLimitCurrent() returns \"{0}\"".format(result))

    # command: getHardwareLimitVoltage
    #     input: -
    #     output: real value
    result = mpod_module.getHardwareLimitVoltage()
    print("getHardwareLimitVoltage() returns \"{0}\"".format(result))

    # command: getIndex
    #     input: -
    #     output: integer value
    result = mpod_module.getIndex()
    print("getIndex() returns \"{0}\"".format(result))

    # command: getMeasurementTemperature0
    #     input: -
    #     output: real value
    result = mpod_module.getMeasurementTemperature0()
    print("getMeasurementTemperature0() returns \"{0}\"".format(result))

    # command: getMeasurementTemperature1
    #     input: -
    #     output: real value
    result = mpod_module.getMeasurementTemperature1()
    print("getMeasurementTemperature1() returns \"{0}\"".format(result))

    # command: getMeasurementTemperature2
    #     input: -
    #     output: Uninitialised
    result = mpod_module.getMeasurementTemperature2()
    print("getMeasurementTemperature2() returns \"{0}\"".format(result))

    # command: getMeasurementTemperature3
    #     input: -
    #     output: real value
    result = mpod_module.getMeasurementTemperature3()
    print("getMeasurementTemperature3() returns \"{0}\"".format(result))

    # command: getRampSpeedCurrent
    #     input: -
    #     output: real value
    result = mpod_module.getRampSpeedCurrent()
    print("getRampSpeedCurrent() returns \"{0}\"".format(result))

    # command: getRampSpeedVoltage
    #     input: -
    #     output: real value
    result = mpod_module.getRampSpeedVoltage()
    print("getRampSpeedVoltage() returns \"{0}\"".format(result))

    # command: getStatus
    #     input: -
    #     output: integer value
    result = mpod_module.getStatus()
    print("getStatus() returns \"{0}\"".format(result))

    # command: getStatusAsString
    #     input: -
    #     output: string value
    result = mpod_module.getStatusAsString()
    print("getStatusAsString() returns \"{0}\"".format(result))

    # command: setDoClear
    #     input: Value
    #     output: True if successfull
    result = mpod_module.setDoClear(1)
    print("setDoClear(1) returns \"{0}\"".format(result))

    # command: setRampSpeedCurrent
    #     input: Value
    #     output: True if successfull
    result = mpod_module.setRampSpeedCurrent(10.5)
    print("setRampSpeedCurrent() returns \"{0}\"".format(result))

    # command: setRampSpeedVoltage
    #     input: Value
    #     output: Uninitialised
    result = mpod_module.setRampSpeedVoltage(0.5)
    print("setRampSpeedVoltage() returns \"{0}\"".format(result))

    # command: getIsHvModule
    #     input: -
    #     output: True or False
    result = mpod_module.getIsHvModule()
    print("getIsHvModule() returns \"{0}\"".format(result))

    ## it takes some time until the the new value is taken into account (write and read to device)
    print("... waiting before comparing read and written value ...")
    time.sleep(wait_delay)
    result = mpod_module.getRampSpeedCurrent()
    print("getRampSpeedCurrent() returns \"{0}\"".format(result))
    result = mpod_module.getRampSpeedVoltage()
    print("getRampSpeedVoltage() returns \"{0}\"".format(result))





thread_polling_interval_ms = 1000.0
wait_delay = 3 * thread_polling_interval_ms * 0.001

# Here we build the relative path to the python module.
# The path might have to be adjusted if the example is run outside the 
# Analog Flavor software installation tree.
mpod_snmp = import_mpod_snmp_module.import_mpod_snmp_module()

# the IP address, port and snmp community can be specified in the first argument as
#           crateConnect("192.168.0.101:161:guru", ...
#       or in the first 3 arguments as
#           crateConnect("192.168.0.101", 161, "guru", ...
# the polling intervals are optional arguments
if (len(sys.argv) >= 2) :
    mpod_crate = mpod_snmp.crateConnect(sys.argv[1], 0, "", 0.5 * thread_polling_interval_ms, thread_polling_interval_ms, 2.0 * thread_polling_interval_ms);
else :
    print("Please specify an MPOD connection on the comnmand line (ex. \"192.168.0.100:161:guru\").")
    default_crate_ip = import_mpod_snmp_module.get_default_crate_ip()
    mpod_crate = mpod_snmp.crateConnect(default_crate_ip, 0, "")

print("newly created crate  == {0}".format(mpod_crate))

# check that the connection has been established 
if (mpod_crate.checkConnectionIsOk()) :

    # wait to make sure all data has been read to the buffer 
    time.sleep(wait_delay)

    all_slots = mpod_crate.getOccupiedSlots()
    print("getOccupiedSlots() returns \"{0}\"".format(all_slots))

    for slot_index in all_slots:
        mpod_module = mpod_crate.getModule(slot_index)
        print("### testing module functions for module \"{0}\"".format(mpod_module))
        test_module_functions(mpod_module)

else :
    # print error messages 
    print("checkConnectionIsOk() returns false.");
    int_value = mpod_crate.getConnectionStatus()
    print("getConnectionStatus() returns {0}.".format(int_value))
    string_value = mpod_crate.getConnectionStatusAsString()
    print("getConnectionStatusAsString() returns {0}.".format(string_value))
    string_value = mpod_crate.getErrorString()
    print("getErrorString() returns {0}.".format(string_value))

exit(0)


